#ifndef   __M_CURVE_H
#define   __M_CURVE_H
// (C) Copyright 1993,95 by Heiko Damerow MPG AG "Roentgenbeugung"

#if defined (Build_Library)
 #define _CURVECLASS _export
#elif defined(Use_Library)
 #define _CURVECLASS _import
#else
 #define _CURVECLASS
#endif


typedef enum 
{
  X_Sorted = 1, Y_Sorted, Z_Sorted, Incoming
} TOrder;

typedef enum 
{
  MaxScanExceeded = 1, NoMemory
} TCFault;


struct CPoint
{
 float Pt[3];
 BOOL  Valid;
};

typedef CPoint * LPCPoint;

class __rtti _CURVECLASS TCurve
{
  public:
             TCurve( int );
             ~TCurve( void );
  BOOL       PAdd( float &x, float &y, float &z );
  BOOL       PGet( float &x, float &y, float &z );
  BOOL       FastPAdd( float &x, float &y, float &z );
  BOOL       FastPGet( float &x, float &y, float &z );
  BOOL       FastOpen( void );
  BOOL       FastClose( void );
  BOOL       ValueAdd( float &val );
  void       BackStep( void );
  BOOL       New( void );
  BOOL       DeleteUnderGround( float Level );
  BOOL       DeleteFlanks();
  BOOL       Save( LPCSTR );
  BOOL       Save( LPCSTR, int, int );
  void       SetLastPoint( void );
  void       SetOrder( TOrder order )
      { eOrder = order; };
  BOOL       SetPP( int pos );
  int        GetId( void )
      { return Id; };
  int        GetPP( void )
      { return ActualIdx; };
  int        GetLastId( void )
      { return LastId; };
  int        GetPNumber( void )
      { return MaxPointIdx+1; };
  BOOL       GetGravityCenter( float&, float&, float&, int );
  int        GetValueByValue( float, float&, float& );
  //! GetReflection wird nirgends benutzt
  //! BOOL       GetReflection( int order, LPCPoint );
  BOOL       GetPeakProperties( float&, float&, float& );
  TCurve&    operator=( const TCurve& );
  float      Min[3], Max[3];
 private:
  HGLOBAL    hMemX;
  HGLOBAL    hMemY;
  HGLOBAL    hMemZ;
  HGLOBAL    hMemV;
  TOrder     eOrder;
  BOOL       bStreamOpen;
  int        Id;            // bezeichnet Kurve
  BOOL       bEnd;          // Ende-Erkennung
  static int LastId;        // Anzahl der genutzten Kurven im Programm
  int        MaxPointIdx;   // indiziert den letzten Eintrag
  int        MaxMemIdx;     // indiziert den maximal moeglichen Eintrag
  int        ActualIdx;     // indiziert den aktuellen Eintrag
  float*     lpPX;          // Zeiger auf Datenfeld von X
  float*     lpPY;          // Zeiger auf Datenfeld von Y
  float*     lpPZ;          // Zeiger auf Datenfeld von Z
  BOOL*      lpPV;          // Zeiger auf Datenfeld von Validate
  float*     lpFastPX;      // Zeiger auf Datenfeld von X
  float*     lpFastPY;      // Zeiger auf Datenfeld von Y
  float*     lpFastPZ;      // Zeiger auf Datenfeld von Z
  BOOL*      lpFastPV;      // Zeiger auf Datenfeld von Validate
};

typedef TCurve FAR * LPCurve;

class __rtti _CURVECLASS TDataBase
{
 public:             
          TDataBase( void );
          ~TDataBase( void );
  void    Reset( void )
      { nActualIdx = 0; };
  int     GetCNumber( void )
      { return nMaxCurveIdx+1; };
  LPCurve GetCurve( int );
  LPCurve GetScanReport( void )
      { return ScanReport; };
  int     GetCIdx( void )
      { return nActualIdx; };
  void    SetCIdx( int );
  BOOL    AddCurve( int size );
  BOOL    New( void );
  int     GetMaxScanNumber( void )
      {
        int cn = nMaxCurve-1;
        return cn;
      };
  TCFault  GetFault( void ) 
      { return eFault; };
 private:
  static const int nMaxCurve;
  TCFault  eFault;
  int      nMaxCurveIdx;  // indiziert die letzte Kurve
  int      nActualIdx;    // indiziert die aktuelle Kurve
  LPCurve* CurveList;
  LPCurve  ScanReport;
};

typedef TDataBase FAR * LPDataBase;

#endif  
//M_CURVE_H
// __LastLine__
