// (C) 1993, 1994 by Heiko Damerow
// MPG AG "Roentgenbeugung an Schichtstrukturen"
#ifndef __M_DEVCOM_H
#define __M_DEVCOM_H

#define F_START             200
#define F_STEP              201
#define F_STOP              202
#define F_INTERRUPT         203
#define F_RESTART           204
#define D_TimeCounting      301
#define D_ImpulseCounting   302
#define D_Intensimeter      303
#define D_Accumulation      304
#define D_GrabPicture       305
#define D_Histogram         306
#define D_DiffractionScan   307
#define D_PSDScan           308
#define D_RRRR              309

#if defined (Build_Counters)
 #define _COUNTERCLASS _export
#elif defined(Use_Counters)
 #define _COUNTERCLASS _import
#else
 #define _COUNTERCLASS
#endif

typedef struct
{
  WPARAM wpJumpTo;
  float  fExposureTime;
  DWORD  dwExposureCounts;
  float  fFailure;
  int    nFunctionId;
  BOOL   bStaticFailure;
  int    nAddedChannels;
} TDSettings;

class __rtti _COUNTERCLASS TDevice
{
  friend class    TDList;
  friend class    TCommonDevParam;
  friend class    TCounterWindow;
  friend class    TAreaScan;
 public:           
                  TDevice( void );
  virtual         ~TDevice( void );
  virtual int     Initialize( void );
  virtual int     SetParameters( void )      
      { return 1; };
  virtual void    SetAddedChannels( int )     
      {};
  virtual int     GetAddedChannels( void )     
      { return 1; };
  virtual float   GetMaximumIntensity( void )   
      { return 1.0; };
  virtual BOOL    SetSound( void )         
      { return TRUE; };
  virtual float   GetAngleRange( void )          
      { return 1.0; }
  virtual float   GetAngleRange( TUnitType )     
      { return 1.0; }
  THardware       GetHardware( void )       
      { return eHardware; };
  virtual int     MeasureStart( void );
  virtual int     MeasureStop( void );
  virtual int     MeasureStopExternal( void )   
      { return 1; };
  int             SetExposureValues( float, DWORD, float );
  void            GetExposureValues( float&, DWORD&, float& );
  virtual void    SetSpezificParametersDlg( void )
      {};
  virtual int     PollDevice( void );
  virtual int     SingleStepMeasurement( int )   
      { return 0; };
  virtual int     GetData( float& );
  virtual int     GetData( TCurve& )        
      { return 0; };
  virtual int     GetData( WORD*, WORD, WORD )    
      { return 0; };
  virtual UINT    GetMaximumChannel( void )    
      { return 0; };
  virtual void    SetAngleStep( float )        
      {};
  virtual float   GetAngleStep( void )       
      { return 0.0; };
  virtual void    SetSignalGrowUp( BOOL )     
      {};
  virtual BOOL    IsSoftOverflow( void )      
      { return FALSE; };
  virtual BOOL    IsHardOverflow( void )      
      { return FALSE; };
  virtual LPCSTR  GetUnitStr( void )       
      { return ""; }
  virtual float   GetWidth( void )         
      { return 1.0; };
  virtual float   GetWidth( TUnitType )      
      { return 1.0; };
  virtual void    SetDataType( TPsdDataType )   
      {};
  inline int      GetId( void )          
      { return nDevId; };
  inline int      IsOk( void )           
      { return bDeviceOk; };
  void            UpdateDisplay( void );
  BOOL            IsActive( void )         
      { return bDeviceActive; };
  void            SetHost( HWND hwnd )      
      { hControlWnd = hwnd; };
  float           GetSigma( void )        
      { return fSigma; };
  virtual BOOL    PushSettings( void );
  virtual BOOL    PopSettings( void );
  BOOL            SelectMFunction( int mf )    
      { nFunctionId = mf; return TRUE; };
  int             GetMFunction( void )       
      { return nFunctionId; };
  static CALLBACK EventHandler( UINT, UINT, DWORD, DWORD, DWORD );
  virtual BOOL    InitializeEvent( HWND, int );
  float           fSigma;
  float           fIntensity;
  BOOL            bDataValid;
  int             nFunctionId;
  int             nPeakWindow;  // in Prozent
  int             nPeakLevel;   // in Prozent
  BOOL            bAnalysator;
  BOOL            bDiskriminator;
  BOOL            bSound;
  BOOL            bDeviceOpen;      // Geraet ist in irgendeiner Form aktiv
  BOOL            bStaticFailure;   // Mess-Fehler auf bestimmtem Wert halten
  char            Characteristic[MaxString];
  double          dRealTime;
 protected:
  void            SetDisplay( HWND hwnd ) 
      { hDisplayWnd = hwnd; };
  HWND            GetDisplay( void )  
      { return hDisplayWnd; };
  HWND            hDisplayWnd;
  HWND            hControlWnd;
  BOOL            bDeviceActive;
  BOOL            bDeviceOk;
  WPARAM          wpJumpTo;
  DWORD           dwElapsedTime, dwStartTime;
  DWORD           dwLowerCountBound;
  DWORD           dwUpperCountBound;
  float           fLowerTimeBound;
  float           fUpperTimeBound;
  UINT            uAskTime;
  BOOL            bDebug;
  double          CalcTime( DWORD );
  HWND            hWndFrame;
  TDSettings      Settings;
  float           fExposureTime;
  DWORD           dwExposureCounts;
  float           fFailure;
  THardware       eHardware;
 private:
  static int      nMaxId;
  int             nDevId;
 };

class  __rtti _COUNTERCLASS TDList
{
 public:
              TDList( int, HINSTANCE );
              ~TDList( void );
  TDevice*    DP( void );
  TDevice*    DP( int );
  TDeviceType ParsingAxis( LPSTR );
  BOOL        SetDevice( int );
  BOOL        IsValidId( int id )
      { return ( ( id >= 0) && (id <= nLastDevice)); };
  BOOL        DeleteDevices( void );
  void        CalibrateDlg( TCurve& );
  inline int  GetDevice( void )
      { return nActiveDevice; };
  inline int  LastId( void )
      { return nLastDevice; };
  inline HINSTANCE GetModuleHandle( void )
      { return hDInstance; };
  BOOL        InitializeModule( void );
  BOOL        SaveModuleSettings( void )
      { return TRUE; };
  void        SetParametersDlg( void );
  int         GetIdByName( TDeviceType );
  BOOL        IsDeviceValid( TDeviceType );
 private:
  int         MD_Monitor;
  int         MD_Encoder;
  int         MD_Psd;
  int         MD_Counter;
  int         MD_Generic;
  HINSTANCE   hDInstance;
  int         nMaxNumber;
  int         nLastDevice;   // points the last device
  int         nActiveDevice;  // points the actual device
  TDevice**   aDevice;
};

class TCounterWindow : public TMDIWindow
{
  friend class TCounterShowParam;
 public:     
             TCounterWindow( int DeviceId );
             ~TCounterWindow( );
  void       Create( HWND hwnd );
  HCURSOR    GetCursor( void ) 
      { return LoadCursor( NULL, IDC_ARROW ); };
  BOOL       bShowAsDigital;
  void       SetupLogging( void );
 protected:
  BOOL       bWriteLogFile;
  TScaleType eScaleType;
  float      fMaxIntensity;
  TDevice*   Sensor;
  void       CounterSetRequest( LPARAM );
  void       Size( HWND, WPARAM, LPARAM );
  void       rButtonDown( LPARAM );
  void       SetFocus( void );
  BOOL       CanOpen( void );
  void       lButtonDown( WPARAM, LPARAM );
  void       Paint( HDC, PAINTSTRUCT * );
 private:
  BOOL       bQuiteOpen;
  HGLOBAL    hMemG;
#if defined(__FLAT__)
  float      *IField;
#else
  float _huge *IField;
#endif
  float      fData;
  int        nIdx;
  int        nMaxBarIdx;
  HPEN       hPen;
  HBRUSH     hBrushWhite, hBrushBlack, hBrushRed, hBrushOC;
  float      fThickness, fHight;
  BOOL       SetTitle( void );
  BOOL       bAnalogSwitchOn;
  BOOL       bWindowWillBeOpen;
  LPCSTR     ClassName() 
      { return "MDevice"; };
  LPCSTR     GetCharacteristic( void ) 
      { return "Device"; };
};

class TCounterShowParam : public TModalDlg
{
 public:
                  TCounterShowParam( TCounterWindow* );
 private:
  TCounterWindow* CounterWnd;
  BOOL            CanClose( void );
  BOOL            Dlg_OnInit( HWND, HWND, LPARAM );
  BOOL            LeaveDialog( void ) 
      { return TRUE; };
};

typedef TDList FAR * LPDList;
typedef TDevice FAR * LPDevice;

#endif
// __M_DEVCOM_H
// __LastLine__
