// File m_mothw.h
// (C) 1993,1994 by Heiko Damerow MPG AG "Roentgenbeugung"
#ifndef __M_MOTHW_H
#define __M_MOTHW_H

#include "C_8x2.inc"

typedef union
{
  char byte[4];
  long pos;
} tpos;

class __rtti TDC_Drive : public TMotor
{
 public:
       TDC_Drive( void );
  // Funktionen, die untergeordnete Pendants aufrufen
  BOOL          GetPosition( BOOL );
  long          GetFailure( void );
  BOOL          MoveByPosition( long );
  BOOL          MoveToPosition( long );
  virtual BOOL  SetMoment( float )         
      { return TRUE; };
  virtual float GetMoment( void )         
      { return 1.0; };
  BOOL          SetSpeed( double );
  double        GetSpeed( void );
  virtual int   SaveSettings( BOOL );
  BOOL          PushSettings( void );
  BOOL          PopSettings( void );
  virtual BOOL  SetVelocity( DWORD )            
      { return TRUE; };
  virtual BOOL  SetAcceleration( DWORD )        
      { return TRUE; };
  virtual DWORD GetVelocity( void )             
      { return 1; };
  virtual DWORD GetAcceleration( void )         
      { return 1; };
 protected: 
  virtual int   Initialize( void );
  virtual BOOL  ActivateDrive( void )           
      { return FALSE; };
  virtual BOOL  Reset( void )                   
      { return FALSE; };
  virtual BOOL  ActivateFilterParameters( void )
      { return FALSE; };
  WORD          wDeathBand;
  WORD          wDynamicFailure;
  WORD          wDeccelerationPoint;
  WORD          wMaxTorque;
  // Spezifische Funktionen
  virtual BOOL  CheckBoardOk( void )            
      { return FALSE; };
  virtual BOOL  IsMoveFinish( void )            
      { return FALSE; };
  virtual BOOL  IsLimitHit( void )              
      { return FALSE; };
  virtual BOOL  _GetPosition( long * )          
      { return FALSE; };
  virtual BOOL  _GetFailure( long * )           
      { return FALSE; };
  virtual BOOL  MoveRelative( long )            
      { return FALSE; };
  virtual BOOL  MoveAbsolute( long )            
      { return FALSE; };
 private:
  int           nMaxError;
};

class __rtti TC_812 : public TDC_Drive
{
  friend class TOptimizeDC_812;
 public:   
               TC_812( void );
  virtual int  ExecuteCmd( char* ) 
      { return R_OK; };
  void         OptimizingDlg( void );
 protected:
  int          SaveSettings( BOOL );
  virtual int  Reset( void );
  virtual BOOL CheckBoardOk( void ) 
      { return TRUE; };
  virtual BOOL SetHome( void );
  virtual BOOL _GetPosition( long* );
  virtual BOOL _GetFailure( long* );
  BOOL         ActivateDrive( void );
  BOOL         ActivateFilterParameters( void );
  BOOL         SetMoment( float );
  float        GetMoment( void );
  virtual BOOL IsMoveFinish( void );
  BOOL         SetLine( int, BOOL );
  BOOL         IsLimitHit( void );
  BOOL         IsIndexArrived( void );
  BOOL         StartToIndex( long& );
  BOOL         StopDrive( int );
  BOOL         MoveRelative( long );
  BOOL         MoveAbsolute( long );
  WORD         GetStatus( void );
  int          nWaitTicks;
  WORD         wTorque;
  WORD         wStaticGain;
  WORD         wDynamicGain;
  int          nOnBoardId;
 private:
  BOOL         SetLimit( DWORD );
  BOOL         SetVelocity( DWORD );
  DWORD        GetVelocity( void );
  BOOL         SetStaticGain( WORD );
  WORD         GetStaticGain( void );
  BOOL         SetDynamicGain( WORD );
  WORD         GetDynamicGain( void );
  BOOL         SetTorque( WORD );
  WORD         GetTorque( void );
  BOOL         SetAcceleration( DWORD );
  DWORD        GetAcceleration( void );
};

class __rtti TC_812ISA : public TC_812
{
 public:
         TC_812ISA( void );
 protected:
  int    ExecuteCmd( char* );
  BOOL   CheckBoardOk( void );
  BOOL   SetHome( void );
  void   StartCheckScan( void );
  // the following functions can used the direct access
  BOOL   IsMoveFinish( void );
  BOOL  _GetPosition( long* );
  BOOL  _GetFailure( long* );
 private:
  int   PutChar( const char );
  char  GetChar( void );
  WORD  wBaseAddr;
  LPSTR lpFlag, lpIn, lpOut1, lpOut2, lpDPRam;
};

class __rtti TC_812GPIB : public TC_812
{
 public:
       TC_812GPIB( void );
 protected:
  int  Initialize( void );
  void StartCheckScan( void );
  int  ExecuteCmd( char* );
  BOOL CheckBoardOk( void );
 private:
  int  Init( void );
  WORD wGPIBAddr;
};


class __rtti TC_832 : public TDC_Drive
{
  friend class TOptimizeDC_832;
  friend long  Drive628c( BYTE, WORD, long, WORD, WORD );
 public:
              TC_832( void );
  BOOL        StopLimitWatch( void );
  BOOL        StartLimitWatch( void );
  void CALLBACK ( *lpfnLimitWatch )( UINT, UINT, DWORD, DWORD, DWORD );
  void        OptimizingDlg( void );
 protected:
  int         SaveSettings( BOOL );
  BOOL        ActivateFilterParameters( void );
  BOOL        CheckBoardOk( void );
  BOOL        ActivateDrive( void );
  void        StartCheckScan( void );
  int         Reset( void );
  WORD        wKI;    //  IntegralGain
  WORD        wKL;    //  IntegralLimit
  WORD        wKP;    //  StaticGain
  WORD        wKD;    //  DynamicGain
 private:
  long        Drive628( BYTE, WORD, long );
  int         ExecuteCmd( char * );
  BOOL        IsMoveFinish( void );
  BOOL        IsLimitHit( void );
  BOOL        IsIndexArrived( void );
  BOOL        StartToIndex( long & );
  BOOL        StopDrive( int );
  BOOL        SetLimit( DWORD );
  BOOL        SetVelocity( DWORD );
  DWORD       GetVelocity( void );
  BOOL        SetHome( void );
  BOOL        SetAcceleration( DWORD );
  DWORD       GetAcceleration( void );
  WORD        GetStatus( void );
  BOOL        _GetPosition( long * );
  BOOL        _GetFailure( long * );
  BOOL        MoveRelative( long );
  BOOL        MoveAbsolute( long );
  // Endlagen-Ueberwachung und Handling
  static void CALLBACK LimitWatch( UINT, UINT, DWORD, DWORD, DWORD );
  static BOOL bLimitHit;
  static BOOL bIdle;
  static BOOL bIOActive;
  WORD        wBaseAddr;
  int         nOnBoardId;
  BYTE        cConfig;
  WORD        wSample;
};

class __rtti TPiezoDrive : public TMotor
{
 public:
  TPiezoDrive() : TMotor()
      {};
};

class __rtti TBurleigh : public TPiezoDrive
{
 public:     
      TBurleigh() 
          {};
 private:
  int nBaseAddr;
};

class TPosControl : public TModalDlg
{
 public:
          TPosControl();
  int     GetBarRange( const TMotor * );
  int     GetBarPos( const TMotor * );
  BOOL    HScrollBar( WPARAM, LPARAM );
  BOOL    Dlg_OnInit( HWND, HWND, LPARAM );
  void    Dlg_OnCommand( HWND, int, HWND, UINT );
  void    Dlg_OnTimer( HWND, UINT );
  BOOL    LeaveDialog( void );
 private:
  long    NewPosition;
  HWND    BarHandle;
  HWND    hMotorList;
  BOOL    bMoveActive;
  LPMotor Motor;
};

class TMotorParam : public TModalDlg
{
 public:
          TMotorParam( void );
  BOOL    Dlg_OnInit( HWND, HWND, LPARAM );
  void    Dlg_OnCommand( HWND, int, HWND, UINT );
  BOOL    LeaveDialog( void ); 
  BOOL    CanClose( void );
 private:
  LPMotor Motor;
  BOOL    Flag;
  float   SpeedScale;
  HWND    hMotorList;
};

class TCalibrate : public TModalDlg
{
 public:
          TCalibrate( int );
          ~TCalibrate( void );
 private:
  BOOL    Dlg_OnInit( HWND, HWND, LPARAM );
  void    Dlg_OnCommand( HWND, int, HWND, UINT );
  void    Dlg_OnTimer( HWND, UINT );
  BOOL    CanClose( void );
  BOOL    LeaveDialog( void );
  HWND    hMotorList;
  LPMotor Motor;
  int     nTask;
  long*   lOldPos;
  BOOL*   bIndexPosReached;
  BOOL*   bZeroPosReached;
  BOOL*   bTargetPosReached;
  BOOL    bValidCalibrationData;
  BOOL    bZeroShouldSet;
  BOOL    bAllMotors;
  BOOL    bHoldPosition;
};

class TOptimizeDC_832 : public TModalDlg
{
 public:         
          TOptimizeDC_832( void );
 private:
  BOOL    CanClose( void );
  void    Dlg_OnCommand( HWND, int, HWND, UINT );
  BOOL    Dlg_OnInit( HWND, HWND, LPARAM );
  BOOL    LeaveDialog( void );
  TC_832* Drive;
  DWORD   old_vel, old_accel;
  WORD    old_kd, old_kp, old_ki, old_kl;
  WORD    old_poswidth;
  BOOL    bCancel;
};

class TOptimizeDC_812 : public TModalDlg
{
 public:        
          TOptimizeDC_812( void );
 private:
  BOOL    CanClose( void );
  void    Dlg_OnCommand( HWND, int, HWND, UINT );
  BOOL    Dlg_OnInit( HWND, HWND, LPARAM );
  BOOL    LeaveDialog( void );
  TC_812* Drive;
  DWORD   old_vel, old_accel;
  WORD    old_gain, old_dgain, old_torque;
  WORD    old_poswidth;
  BOOL    bCancel;
};
#endif
// _M_MOTHW_H
// __LastLine__

