// File m_psd.h
// (C) 1993,1994 by Heiko Damerow MPG AG "Roentgenbeugung an Schichtstrukturen"
#ifndef __M_PSD_H
#define __M_PSD_H

#define IO_Wait_Psd 27
#include "stoe_psd.inc"

#if defined (Build_Counters)
 #define _COUNTERCLASS _export
#elif defined(Use_Counters)
 #define _COUNTERCLASS _import
#else
 #define _COUNTERCLASS
#endif

class __rtti TPsd : public TDevice
{
  friend class  TPsdParameters;
 public:      
              TPsd( void );
  virtual     ~TPsd( void );
  virtual int Initialize( void );
  BOOL        PushSettings( void );
  BOOL        PopSettings( void );
  virtual int SetEnergyRange( UINT, UINT )   
      { return R_OK; };
  virtual int GetEnergyRange( UINT&ler, UINT& her )
      { her = 1; ler = 0; return R_OK; };
  void        SetAddedChannels( int ach );
  void        SetSpezificParametersDlg( void );
  void        SetAngleRange( float );
  int         SetParameters( void );
  void        SetSignalGrowUp( BOOL gup )   
      { bSignalGrowUp = gup; };
  void        SetAngleStep( float stp )    
      { fAngleStep = stp; };
  virtual int GetBufferSize( void )      
      { return nChannelNumber+1; };
  float       GetAngleStep( void )       
      { return fAngleStep; };
  float       GetAngleRange( void );
  float       GetAngleRange( TUnitType );
  BOOL        IsSoftOverflow( void )      
      { return bSoftOverflow; };
  BOOL        IsHardOverflow( void )      
      { return bHardOverflow; };
  float       GetWidth( TUnitType );
  float       GetWidth( void )         
      { return fAngleWidth; };
  int         GetAddedChannels( void )     
      { return nAddedChannels; };
  int         GetChannelNumber( void )     
      { return nChannelNumber; };
  int         GetData( TCurve& );
  int         GetData( float& );
  int         GetData( WORD*, WORD, WORD )    
      { return 0; };
  UINT        GetMaximumChannel( void );
  float       GetMaximumIntensity( void )   
      { return ( dwMaxCounts / dRealTime ); };
  virtual int MeasureStart( void );
  virtual int MeasureStop( void );
  int         MeasureStopExternal( void );
  virtual int PollDevice( void );
  LPCSTR      GetUnitStr( void )       
      { return (LPCSTR)szUnit; }
 protected:
  char        szUnit[20];
  int         nBaseAddr;
  TUnitType   eUnit;
  HGLOBAL     hCountBuf;
  BOOL        bHVRegelung;          // boolscher Wert
  int         nCountBufItems;
  int         nMinChannel;
  int         nMaxChannel;
  int         nChannelNumber;
  DWORD       dwMaxCounts, dwIntegratedCounts;
  UINT        uMaximumChannel;
  BOOL        bReadyForRead;
  BOOL        bHardOverflow;
  BOOL        bReadLeftFirst;
  long        lSeconds;
  long        lWaitTime;
  BOOL        bFirstReadOut;
  BOOL        bSignalGrowUp;
 private:
  float       fAngleStep;
  float       fAngleWidth;
  int         nAddedChannels;
  int         s_nAddedChannels;
  float       s_AngleStep;
  virtual int PsdInit( void )   
      { return R_OK; };
  virtual int PsdStart( void )   
      { return R_OK; };
  virtual int PsdStop( void )   
      { return R_OK; };
  virtual int PsdReadOut( THowReadOutPsd );
  virtual int PsdRead( int, int, int, LPWORD ) 
      { return R_OK; };
  BOOL        bSoftOverflow;
  float       OverflowIntensity;
};

class TStoe_Psd : public TPsd
{
 public:
          TStoe_Psd( void );
          ~TStoe_Psd( void );
  int     PollDevice( void );
  int     GetBufferSize( void ) 
      { return nMaxChannel-nMinChannel+1; };
 private:
  BOOL    bLong;
  int     Initialize( void );
  int     SetParameters( void ) 
      { return R_OK; };
  int     PsdReadOut( THowReadOutPsd );
  int     PsdRead( int, int, int, LPWORD );
  int     PsdInit( void );
  int     PsdStart( void );
  int     PsdStop( void );
  HGLOBAL hReadBuf;
  int     nReadBufItems;
};

class TBraun_Psd : public TPsd
{
 public:
       TBraun_Psd( void );
       ~TBraun_Psd( void );
  int  Initialize( void );
  //int  PollDevice( void );
  int  PsdInit( void );
  int  PsdStart( void );
  int  SetParameters( void )  
      { return R_OK; };
  int  PsdReadOut( THowReadOutPsd );
  int  GetBufferSize( void );
  void SetDataType( TPsdDataType );
  int  SetEnergyRange( UINT, UINT );
  int  GetEnergyRange( UINT& ler, UINT& her )
      { her = uEnergyHigh; ler = uEnergyLow; return R_OK; };
  int  PsdStop( void );
  int  nErrorCode;
  BOOL bSetError;
  int  nHVRegelung_OK;
 protected:
  int  BuildOperation( BYTE*, BYTE*, int );
  int  LoadHexFile( void );
  void ResetDelayTime( void );
  BYTE konvert( char );
  int  SynchronHexFile( BYTE&, BYTE& );
  int  Look_till_BaseAddr1( int );
 private:
  TPsdDataType eDataType;
  typedef union
  {
    LONG LWert;
    WORD IWert[2];
    BYTE SWert[4];
  } long_to_bytes;
  struct   MessDatenHeader
  {
    long HighADCCounts;
    long LowADCCounts;
    long HighEGYCounts;
    long LowEGYCounts;
    long HighPOSCounts;
    long LowPOSCounts;
    long HSumDetCounts;
    long LSumDetCounts;
    long RateADC;
    long RateEGY;
    long RatePOS;
    long RateDET;
    long Messzeit;
    long HVParameter;
    long EndLifeTime;
    long Ueberlauf;
  }  Header;
  static BYTE echo[30][16];
  static BYTE befehl[30][16];
  char        Buffer[16];
  UINT        uMuxTimeDet1;
  LONG        PositionsDatenHeader[16];
  UINT        uEnergyScale;         // moeglich sind 00, 01, 02, 03 hex
  UINT        uPositionScale;       // moeglich sind 00, 01, 02, 03 hex
  UINT        uEnergyHigh;
  UINT        uEnergyLow;
  LONG        lPositionStop;        // Countstop fuer Positionskanaele
                                    // 0..EFFFFFFFFFFF hex
  BOOL        bDebugInfo;           // boolscher Wert
  BOOL        bAbbruchmitShutter;   // boolscher Wert
  int         nDeathTime;     // min. Impulsabstand  0..99 dec [0..99 *E-6 sec]
  BOOL        bRatemeter;           // computed=00,  total=01
  BOOL        bRealLifeTime;        // Realtime=00,  Lifetime=01
  LONG        lMeasTime;            // Messzeit 1..9999999 dec [sec]
  HGLOBAL     hReadBuf;
  int         nDelayFast;
  int         nDelaySlow;
  UINT        uCBufferLength;
  int         nReadBufItems;
};

class TPsdParameters : public TModelessDlg
{
 public:
        TPsdParameters( void );
 private:
  TPsd* Device;
  int   OldId;
  BOOL  Dlg_OnInit( HWND, HWND, LPARAM );
  void  Dlg_OnCommand( HWND, int, HWND, UINT );
  BOOL  CanClose( void );
  BOOL  LeaveDialog( void );
};
#endif
// __M_PSD_H
// __LastLine__
